#!/usr/bin/env python

# -*- coding: utf-8 -*-

from __future__ import absolute_import

from __future__ import print_function

from __future__ import division

from traits.api import HasTraits, Instance, Property, Enum, Button,String,\
                       on_trait_change, Float, Bool, Int,Constant, ReadOnly,\
                       List, Array, Range, Event

from traitsui.api import View, Item, HSplit, VSplit, InstanceEditor, HGroup,\
                         Group, ListEditor, Tabbed, VGroup, CheckListEditor,\
                         ArrayEditor, Action, ToolBar, Separator, Controller

from tvtk.pyface.scene_editor import SceneEditor

from mayavi.tools.mlab_scene_model import MlabSceneModel

from pyface.image_resource import ImageResource

#------------------------------------------------------------------------------

from geolab.gui.scenemanager import SceneManager

from geolab.gui.multiscenemanager import MultiSceneManager

from geolab.plotter.geolabscene import GeolabScene

from geolab.gui.geolabcomponent import GeolabComponent

from geolab.optimization.gridshell import Gridshell

from geolab.gui.tools import IncrementalRemesh, CornerTolerance,\
                                  Loads, SaveGeometry, Import

from geolab.gui.moviemanager import MovieManager

# -----------------------------------------------------------------------------

__author__ = 'Davide Pellis'


# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#                                     Handler
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------


class GlHandler(Controller):

    def close(self, info, is_ok):
        info.object._closing()
        info.object._closed = True
        Controller.close(self ,info, is_ok)
        return True

    #--------------------------------------------------------------------------
    #                                  Tools
    #--------------------------------------------------------------------------

    def open_plot_manager(self):
        self.info.object.open_plot_manager()
        
    def open_movie_manager(self):
        self.info.object.open_movie_manager()

    def open_corner_tolerance(self):
        self.info.object.open_corner_tolerance()

    def open_save_geometry(self):
        self.info.object.open_save_geometry()

    def open_remesh(self):
        self.info.object.open_remesh()

    def open_loads(self):
        self.info.object.open_loads()
        
    def open_import(self):
        self.info.object.open_import()


    #--------------------------------------------------------------------------
    #                                 Selection
    #--------------------------------------------------------------------------

    def background_switch(self):
        self.info.object.background_switch()

    def select_object(self):
        self.info.object.select_object()

    def select_vertices(self):
        self.info.object.select_vertices()

    def select_edges(self):
        self.info.object.select_edges()

    def select_faces(self):
        self.info.object.select_faces()

    def select_boundary_vertices(self):
        self.info.object.select_boundary_vertices()
        
    def select_all_vertices(self):
        self.info.object.select_all_vertices()

    def move_vertices(self):
        self.info.object.move_vertices()

    #--------------------------------------------------------------------------
    #                                 Mesh state
    #--------------------------------------------------------------------------

    def reset_mesh(self):
        self.info.object.reset_mesh()

    def set_reference(self):
        self.info.object.set_reference()
        
    def delete_selected_object(self):
        self.info.object.delete_selected_object()

    #--------------------------------------------------------------------------
    #                                 Mesh Edit
    #--------------------------------------------------------------------------

    def flip_edges(self):
        self.info.object.flip_edges()

    def collapse_edges(self):
        self.info.object.collapse_edges()

    def split_edges(self):
        self.info.object.split_edges()
        
    def delete_edges(self):
        self.info.object.delete_edges()

    def catmull_clark(self):
        self.info.object.catmull_clark()

    def loop(self):
        self.info.object.loop()

    #--------------------------------------------------------------------------
    #                              Set conditions
    #--------------------------------------------------------------------------

    def fix_vertices(self):
        self.info.object.fix_vertices()

    def unfix_vertices(self):
        self.info.object.unfix_vertices()

    def constrain_vertices(self):
        self.info.object.constrain_vertices()
        
    def release_vertices(self):
        self.info.object.release_vertices()


# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#                       The Base Graphic User Interface
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------


class GeolabGUI(MultiSceneManager):

    _handler = GlHandler()

    _closed = Bool(True)

    _current_object = String('none')

    _components = []

    scene_model_0 = Instance(MlabSceneModel, ())

    scene_model_1 = Instance(MlabSceneModel, ())

    scene_model_2 = Instance(MlabSceneModel, ())

    scene_model_3 = Instance(MlabSceneModel, ())

    components = List(GeolabComponent)

    remesh_tool = IncrementalRemesh()
    
    movie_manager = MovieManager()
    
    import_tool = Import()

    corner_tolerance_tool = CornerTolerance()

    loads_tool = Loads()

    save_geometry_tool = SaveGeometry()

    _background_switch_counter = Int(1)

    background_switch_button = Action(action = 'background_switch',
                               image = ImageResource('../icons/background.png'),
                               style = 'push',
                               tooltip = 'Switch background color',
                               show_label = False)

    save_mesh_button = Action(action = 'open_save_geometry',
                       image = ImageResource('../icons/save.png'),
                       style = 'push',
                       tooltip = 'Save mesh',
                       show_label = False)
    
    import_button = Action(action = 'open_import',
                       image = ImageResource('../icons/open.png'),
                       style = 'push',
                       tooltip = 'Import',
                       show_label = False)
    
    movie_manager_button = Action(action = 'open_movie_manager',
                           image=ImageResource('../icons/moviemanager.png'),
                           style = 'push',
                           tooltip = 'Movie studio',
                           show_label=False)

    plot_manager_button = Action(action = 'open_plot_manager',
                          image=ImageResource('../icons/plotmanager.png'),
                          style = 'push',
                          tooltip = 'Object plot settings',
                          enabled_when = ('_current_object == \
                                          "Mesh_plot_manager" \
                                          or _current_object == \
                                          "Points_plot_manager"'),
                          show_label=False)

    corner_tolerance_button = Action(action = 'open_corner_tolerance',
                              image = ImageResource('../icons/corners.png'),
                              style = 'push',
                              tooltip = 'Set corner tolerance',
                              enabled_when = ('_current_object == \
                                              "Mesh_plot_manager"'),
                              show_label=False)

    select_object_button = Action(action = 'select_object',
                           image = ImageResource('../icons/selectobject.png'),
                           style = 'toggle',
                           tooltip = 'Select object',
                           show_label = False)

    flip_edges_button = Action(action = 'flip_edges',
                        image = ImageResource('../icons/flip.png'),
                        style = 'toggle',
                        tooltip = 'Flip mesh edges',
                        enabled_when = ('_current_object == \
                                        "Mesh_plot_manager"'),
                        show_label=False)

    split_edges_button = Action(action='split_edges',
                         image = ImageResource('../icons/split.png'),
                         style = 'toggle',
                         enabled_when = ('_current_object == \
                                         "Mesh_plot_manager"'),
                         tooltip = 'Split mesh edges',
                         show_label = False)

    collapse_edges_button = Action(action='collapse_edges',
                            image=ImageResource('../icons/collapse.png'),
                            style='toggle',
                            enabled_when =('_current_object == \
                                           "Mesh_plot_manager"'),
                            tooltip = 'Collapse mesh edges',
                            show_label=False)
                            
    delete_edges_button = Action(action='delete_edges',
                            image=ImageResource('../icons/deledge.png'),
                            style='toggle',
                            enabled_when =('_current_object == \
                                           "Mesh_plot_manager"'),
                            tooltip = 'Delete mesh edges',
                            show_label=False)

    catmull_clark_button = Action(action = 'catmull_clark',
                           image = ImageResource('../icons/catmullclark.png'),
                           style = 'push',
                           enabled_when = ('_current_object == \
                                           "Mesh_plot_manager"'),
                           tooltip = 'Catmull-Clark subdivision',
                           show_label = False)

    loop_button = Action(action = 'loop',
                  image = ImageResource('../icons/loop.png'),
                  style = 'push',
                  enabled_when = ('_current_object == "Mesh_plot_manager"'),
                  tooltip = 'Loop subdivision',
                  show_label = False)

    remesh_button = Action(action = 'open_remesh',
                    image = ImageResource('../icons/remesh.png'),
                    style = 'push',
                    enabled_when = ('_current_object == "Mesh_plot_manager"'),
                    tooltip = 'Incremental remesh',
                    show_label = False)

    move_vertices_button = Action(action = 'move_vertices',
                           image = ImageResource('../icons/movevertex.png'),
                           style = 'toggle',
                           tooltip = 'Move vertices',
                           enabled_when = ('_current_object == \
                                           "Mesh_plot_manager" \
                                           or _current_object == \
                                           "Points_plot_manager"'),
                           show_label=False)

    select_vertices_button = Action(action='select_vertices',
                             image=ImageResource('../icons/selectvertices.png'),
                             style='toggle',
                             tooltip = 'Select vertices',
                             enabled_when = ('_current_object == \
                                             "Mesh_plot_manager" \
                                             or _current_object == \
                                             "Points_plot_manager"'),
                             show_label=False)

    select_edges_button = Action(action='select_edges',
                          image=ImageResource('../icons/selectedges.png'),
                          style='toggle',
                          enabled_when =('_current_object=="Mesh_plot_manager"'),
                          tooltip = 'Select edges',
                          show_label=False)

    select_faces_button = Action(action='select_faces',
                          image=ImageResource('../icons/selectfaces.png'),
                          style='toggle',
                          enabled_when =('_current_object == \
                                         "Mesh_plot_manager"'),
                          tooltip = 'Select faces',
                          show_label=False)

    select_boundary_vertices_button = Action(action='select_boundary_vertices',
                                   image=ImageResource('../icons/boundary.png'),
                                   style='toggle',
                                   enabled_when =('_current_object == \
                                                  "Mesh_plot_manager"'),
                                   tooltip = 'Select boundary vertices',
                                   show_label=False)
                                   
    select_all_vertices_button = Action(action='select_all_vertices',
                                 image=ImageResource('../icons/allvertices.png'),
                                 style='toggle',
                                 enabled_when =('_current_object == \
                                                "Mesh_plot_manager" \
                                                or _current_object == \
                                                "Points_plot_manager"'),
                                 tooltip = 'Select all vertices',
                                 show_label=False)

    fix_vertices_button = Action(action='fix_vertices',
                          image=ImageResource('../icons/fixvertices.png'),
                          style='push',
                          tooltip = 'Fix selected vertices',
                          show_label=False)

    unfix_vertices_button = Action(action='unfix_vertices',
                            image=ImageResource('../icons/unfixvertices.png'),
                            style='push',
                            tooltip = 'Unfix selected vertices',
                            show_label=False)

    constrain_vertices_button = Action(action='constrain_vertices',
                                image=ImageResource('../icons/constrain.png'),
                                style='push',
                                tooltip = 'Constrain selected vertices',
                                show_label=False)

    release_vertices_button = Action(action='release_vertices',
                                image=ImageResource('../icons/unconstrain.png'),
                                style='push',
                                tooltip = 'Release selected vertices',
                                show_label=False)

    loads_button = Action(action='open_loads',
                   image=ImageResource('../icons/applyforce.png'),
                   style='push',
                   tooltip = 'loads_tool',
                   show_label=False)

    reset_mesh_button = Action(action='reset_mesh',
                        image=ImageResource('../icons/resetmesh.png'),
                        style='push',
                        tooltip = 'Reset mesh',
                        show_label=False)

    set_reference_button = Action(action='set_reference',
                           image=ImageResource('../icons/setreference.png'),
                           style='push',
                           tooltip = 'Set as reference mesh',
                           show_label=False)
    
    delete_selected_object_button = Action(action='delete_selected_object',
                           image=ImageResource('../icons/delobject.png'),
                           style='push',
                           tooltip = 'Delete object',
                           show_label=False)

    __toolbar = [Separator(),
                 plot_manager_button,

                 Separator(),
                 select_object_button,
                 select_vertices_button,
                 select_edges_button,
                 select_faces_button,
                 move_vertices_button,
                 select_all_vertices_button,
                 select_boundary_vertices_button,
                 corner_tolerance_button,

                 Separator(),
                 fix_vertices_button,
                 unfix_vertices_button,
                 constrain_vertices_button,
                 release_vertices_button,
                 loads_button,

                 Separator(),
                 collapse_edges_button,
                 flip_edges_button,
                 split_edges_button,
                 delete_edges_button,
                 catmull_clark_button,
                 loop_button,
                 remesh_button,

                 Separator(),
                 reset_mesh_button,
                 set_reference_button,
                 delete_selected_object_button,
                 
                 Separator(),
                 import_button,
                 save_mesh_button,
                 #movie_manager_button,

                 Separator(),
                 background_switch_button,
                 ]

    toolbar = { 'handler': _handler,
                'resizable' : True,
                'title' : 'GeoLab',
                'icon' : ImageResource('../icons/geolab_logo.png'),
                'toolbar' : ToolBar(*__toolbar,
                                    show_labels=False,
                                    image_size=(16,16)),
              }



    tabs = [Item('components',
                 editor=ListEditor(use_notebook=True, page_name='.name'),
                 style ='custom',
                 width = 1,
                 resizable = False,
                 show_label=False)
           ]

    __view3D = []

    __windows = []

    height = Int(200)

    width = Int(200)

    side_width = Int(200)

    #--------------------------------------------------------------------------
    #
    #--------------------------------------------------------------------------

    def __init__(self):
        MultiSceneManager.__init__(self)

        self.__scenes = ['scene_0']

        self.__geometries = []

        self.__object_open_callbacks = []

        self.__initialize_plot = True
        
        self.__change = False

        self._scene_model = self.scene_model_0

    @property
    def geometries(self):
        return self.__geometries
    
    @property
    def movie_studio(self):
        return self.movie_manager

    #--------------------------------------------------------------------------
    #                             Building up
    #--------------------------------------------------------------------------

    def start(self):
        self.make_scenes()
        if len(self._components) > 0 and len(self.__windows) > 0:
            self.components = self._components
            view = View(HSplit(self.tabs,
                               self.__view3D,
                               self.__windows),
                        **self.toolbar)
        elif len(self._components) > 0:
            self.components = self._components
            view = View(HSplit(self.tabs,
                               self.__view3D),
                        **self.toolbar)
        else:
            view = View(self.__view3D, **self.toolbar)
        self._closed = False
        for component in self._components:
            component.initialize_plot()
        for geometry in self.__geometries:
            self.add_object(geometry)
        self.object_changed()
        self.update_plot()
        self.configure_traits(view=view)

    def add_component(self, component):
        component.geolab = self
        self._components.append(component)
        component.geolab_settings()
        self.__initialize_plot = False

    def add_scene(self, name):
        if name not in self.__scenes:
            self.__scenes.append(name)

    def make_scenes(self):
        index = 0
        for key in self.__scenes:
            if index == 0:
                scene = self.scene_model_0
                self.__view3D = [Item('scene_model_0',
                                 editor = SceneEditor(scene_class=GeolabScene),
                                 show_label = False,
                                 resizable = True,
                                 height = self.height,
                                 width = self.width)]
            else:
                if index == 1:
                    scene = self.scene_model_1
                elif index == 2:
                    scene = self.scene_model_2
                elif index == 3:
                    scene = self.scene_model_3
                else:
                    return
                name = ('scene_model_{}').format(index)
                self.__windows.append(Item(name,
                                 editor = SceneEditor(scene_class=GeolabScene),
                                 show_label = False,
                                 resizable = True,
                                 height = 100,
                                 width = self.side_width))
            self.add_scene_model(scene, key)
            index += 1
        if index <= 3:
            self.scene_model_3.mayavi_scene.remove()
        if index <= 2:
            self.scene_model_2.mayavi_scene.remove()
        if index <= 1:
            self.scene_model_1.mayavi_scene.remove()
            
    def _closing(self):
        for component in self._components:
            component.geolab_closing()      

    #--------------------------------------------------------------------------
    #                             Standard Methods
    #--------------------------------------------------------------------------        

    def set_state(self, name):
        self.select_off()
        if name != 'select_object':
            if self.select_object_button.checked:
                self.select_object_off()
            self.select_object_button.checked = False            
        if name != 'flip_edges':
            self.flip_edges_button.checked = False
        if name != 'split_edges':
            self.split_edges_button.checked = False
        if name != 'collapse_edges':
            self.collapse_edges_button.checked = False
        if name != 'delete_edges':
            if self.delete_edges_button.checked:
                self.object_changed()
            self.delete_edges_button.checked = False
        if name != 'move_vertices':
            self.move_vertices_button.checked = False
        if name != 'select_vertices':
            self.select_vertices_button.checked = False
        if name != 'select_faces':
            self.select_faces_button.checked = False
        if name != 'select_edges':
            self.select_edges_button.checked = False
        if name != 'select_boundary_vertices':
            self.select_boundary_vertices_button.checked = False
        if name != 'select_all_vertices':
            self.select_all_vertices_button.checked = False
        for component in self._components:
            component.geolab_set_state(name)
            
    def object_change(self):
        SceneManager.object_change(self)      

    def object_changed(self):
        SceneManager.object_changed(self)
        self.close_tools()
        self._current_object = self.current_object_type

    #--------------------------------------------------------------------------
    #                                Background
    #--------------------------------------------------------------------------

    def background_switch(self):
        if self._background_switch_counter == 1:
            self.set_background((1, 1, 1))
            self._background_switch_counter = 0
        else:
            self.set_background((0.5, 0.5, 0.5))
            self._background_switch_counter = 1

    #--------------------------------------------------------------------------
    #                                  Open
    #--------------------------------------------------------------------------

    def open_obj_file(self, file_name):
        M = Gridshell(file_name)
        self.__geometries.append(M)
        if not self._closed:
            self.add_object(M)

    def open_geometry(self, geometry):
        self.__geometries.append(geometry)
        if not self._closed:
            self.add_object(geometry)

    def reset_mesh(self):
        self.set_state(None)
        self.current_object.geometry.reset()
        self.current_object.update_plot()
        self.object_changed()

    def set_reference(self):
        self.current_object.geometry.set_reference()
        
    def delete_selected_object(self):
        self.set_state(None)
        self.delete_object(self.current_object.name)
        
    def add_object(self, obj, name=None, scene=None):
        self.set_state(None)
        MultiSceneManager.add_object(self, obj, name, scene)

    #--------------------------------------------------------------------------
    #                                  Tools
    #--------------------------------------------------------------------------

    def open_plot_manager(self):
        self.selection_off()
        self.current_object.start()
        
    def open_movie_manager(self):
        #self.set_state(None)
        if self.movie_manager.geolab is None:
            self.movie_manager.geolab = self
        self.movie_manager.start()
        
    def open_import(self):
        self.set_state(None)
        self.import_tool.scenemanager = self
        self.import_tool.start()

    def open_corner_tolerance(self):
        self.set_state(None)
        self.corner_tolerance_tool.scenemanager = self
        self.corner_tolerance_tool.start()

    def open_save_geometry(self):
        self.save_geometry_tool.scenemanager = self
        self.save_geometry_tool.start()

    def open_remesh(self):
        self.set_state(None)
        self.remesh_tool.scenemanager = self
        self.remesh_tool.start()

    def open_loads(self):
        self.loads_tool.scenemanager = self
        self.loads_tool.start()

    @on_trait_change('_closed')
    def close_all(self):
        self.close_tools()
        self.movie_manager.close()
        
    def close_tools(self):
        try:
            self.current_object.close()
        except:
            pass
        self.corner_tolerance_tool.close()
        self.remesh_tool.close()
        self.loads_tool.close()
        self.save_geometry_tool.close()
        self.import_tool.close()


    #--------------------------------------------------------------------------
    #                                Selection
    #--------------------------------------------------------------------------

    def selection_off(self):
        MultiSceneManager.select_off(self)
        self.select_boundary_vertices_button.checked = False
        self.select_edges_button.checked = False
        self.select_object_button.checked = False
        self.select_vertices_button.checked = False
        self.move_vertices_button.checked = False

    def select_object(self):
        if self.select_object_button.checked:
            self.set_state('select_object')
            super(GeolabGUI, self).select_object()
        else:
            self.select_object_off()

    def select_vertices(self):
        if self.select_vertices_button.checked:
            self.set_state('select_vertices')
            self.current_object.select_vertices()
        else:
            self.current_object.select_vertices_off()

    def select_edges(self):
        if self.select_edges_button.checked:
            self.set_state('select_edges')
            self.current_object.select_edges()
        else:
            self.current_object.select_edges_off()

    def select_faces(self):
        if self.select_faces_button.checked:
            self.set_state('select_faces')
            self.current_object.select_faces()
        else:
            self.current_object.select_faces_off()

    def select_boundary_vertices(self):
        if self.select_boundary_vertices_button.checked:
            self.set_state('select_boundary_vertices')
            self.current_object.select_boundary_vertices()
        else:
            self.current_object.select_boundary_vertices_off()
            
    def select_all_vertices(self):
        if self.select_all_vertices_button.checked:
            
            self.set_state('select_all_vertices')
            self.current_object.select_all_vertices()
        else:
            self.current_object.select_vertices_off()

    def move_vertices(self):
        if self.move_vertices_button.checked:
            self.set_state('move_vertices')
            def callback():
                self.current_object.update_plot()
            self.current_object.move_vertices(callback)
        else:
            self.current_object.move_vertices_off()

    #--------------------------------------------------------------------------
    #                                Mesh Edit
    #--------------------------------------------------------------------------

    def flip(self, edge_index):
        self.fix_edges_bug()
        self.current_object.mesh.flip_edge(edge_index)
        #self.object_changed()
        self.current_object.update_plot()

    def collapse(self, edge_index):
        self.fix_edges_bug()
        self.current_object.mesh.collapse_edge(edge_index)
        #self.object_changed()
        self.current_object.update_plot()

    def split(self, edge_index):
        self.fix_edges_bug()
        self.current_object.mesh.split_edge(edge_index)
        #self.object_changed()
        self.current_object.update_plot()
        
    def del_edges(self, edge_index):
        self.fix_edges_bug()
        self.current_object.mesh.delete_edge(edge_index)
        #self.object_changed()
        self.current_object.update_plot()

    def flip_edges(self):
        if self.flip_edges_button.checked:
            self.set_state('flip_edges')
            self.current_object.on_edge_selection(self.flip)

        else:
            self.current_object.select_edges_off()

    def collapse_edges(self):
        if self.collapse_edges_button.checked:
            self.set_state('collapse_edges')
            self.current_object.on_edge_selection(self.collapse)
            self.object_changed()
        else:
            self.current_object.select_edges_off()

    def split_edges(self):
        if self.split_edges_button.checked:
            self.set_state('split_edges')
            self.current_object.on_edge_selection(self.split)
        else:
            self.current_object.select_edges_off()
            
    def delete_edges(self):
        if self.delete_edges_button.checked:
            self.set_state('delete_edges')
            self.current_object.on_edge_selection(self.del_edges)
        else:
            self.current_object.select_edges_off()
            self.object_changed()

    def catmull_clark(self):
        self.set_state(None)
        self.fix_edges_bug()
        self.current_object.mesh.catmull_clark()
        self.object_changed()
        self.current_object.update_plot()

    def loop(self):
        self.set_state(None)
        self.fix_edges_bug()
        self.current_object.mesh.loop()
        self.object_changed()
        self.current_object.update_plot()

    def fix_edges_bug(self):
        obj = self.current_object
        if True:#obj.mesh.E > 2000:
            if obj.edge_plot != 'none':
                if obj.edge_plot != 'wireframe':
                    obj.remove_edges()
                    obj.remove_faces()

    #--------------------------------------------------------------------------
    #                              Set conditions
    #--------------------------------------------------------------------------

    def fix_vertices(self):
        selected = self.current_object.selected_vertices
        self.current_object.mesh.fix(selected)
        self.current_object.update_plot()

    def unfix_vertices(self):
        selected = self.current_object.selected_vertices
        self.current_object.mesh.unfix(selected)
        self.current_object.update_plot()

    def constrain_vertices(self):
        selected = self.current_object.selected_vertices
        self.current_object.mesh.constrain(selected)
        self.current_object.update_plot()
        
    def release_vertices(self):
        selected = self.current_object.selected_vertices
        self.current_object.mesh.release(selected)
        self.current_object.update_plot()


# -----------------------------------------------------------------------------

def view(objects, position=None):
    viewer = GeolabGUI()
    #viewer.position = position
    viewer.add(objects)
    viewer.start()
