# -*- coding: utf-8 -*-

from __future__ import absolute_import

from __future__ import print_function

from __future__ import division

from traits.api import HasTraits, Instance

from traitsui.api import View, Item

from tvtk.pyface.scene_editor import SceneEditor

from mayavi.tools.mlab_scene_model import MlabSceneModel

from pyface.image_resource import ImageResource

# -----------------------------------------------------------------------------

from geolab.plotter.geolabscene import GeolabScene

from geolab.plotter.plotmanager import PlotManager

# -----------------------------------------------------------------------------

'''viewer.py: The viewer for plotter souce classes'''

__author__ = 'Davide Pellis'

# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#                                    Viewer
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------


class Viewer(HasTraits):

    # -------------------------------------------------------------------------
    #                                  Traits
    # -------------------------------------------------------------------------

    scene = Instance(MlabSceneModel, ())

    editor = SceneEditor(scene_class=GeolabScene)

    # -------------------------------------------------------------------------
    #                                   View
    # -------------------------------------------------------------------------

    view = View(Item('scene',
                     editor=editor,
                     show_label=False,
                     resizable=True,
                     height=200,
                     width=200,
                     ),
                resizable=True,
                title = 'GeoLab viewer',
                icon = ImageResource('../icons/geolab_logo.png')
                )

    # -------------------------------------------------------------------------
    #                                 Initialize
    # -------------------------------------------------------------------------

    def __init__(self, objects):
        HasTraits.__init__(self)

        self._plotmanager = PlotManager(scene_model=self.scene)

        self._generate_data(objects)

        self._position = None

    @property
    def position(self):
        return self.editor.scene_class._position

    @position.setter
    def position(self, position):
        self.editor.scene_class._position = position

    @property
    def background(self):
        return self._plotmanager.background

    @background.setter
    def background(self, background):
        self._plotmanager.background = background

    # -------------------------------------------------------------------------
    #                                 Methods
    # -------------------------------------------------------------------------

    def start(self):
        self.configure_traits()

    def _generate_data(self, objects):
        self._plotmanager.add(objects, rename=True)

# -----------------------------------------------------------------------------
#                                 View Function
# -----------------------------------------------------------------------------

def view(objects, position=None, background=(1,1,1)):
    viewer = Viewer(objects)
    viewer.background = background
    viewer.position = position
    viewer.start()
